module MIME

  #
  # Parse MIME messages, constructing Media objects for each MIME entity.
  #
  # TODO Implement
  #
  class Parser

    def initialize
      raise NotImplementedError.new('building a parser is a TODO')
    end

  end

end
